"""Stela exceptions."""


class StelaEnvironmentNotFoundError(Exception):
    """Environment not found Error."""

    pass


class StelaFileTypeError(Exception):
    """File Type Error."""

    pass


class StelaTooManyLoadersError(Exception):
    """Too Many Loaders Error."""

    pass
